package com.nisum.products_producer.controller;

import com.nisum.products_producer.dto.ProductDto;

import com.nisum.products_producer.exception.InvalidPriceException;
import com.nisum.products_producer.exception.InvalidProductNameException;
import com.nisum.products_producer.model.Product;
import com.nisum.products_producer.service.ProductService;
import com.nisum.products_producer.service.ProductServiceImpl;
import jakarta.validation.Valid;
import jakarta.validation.ValidationException;
import jakarta.validation.Validator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BeanPropertyBindingResult;
import org.springframework.validation.BindingResult;
import org.springframework.validation.Errors;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.support.WebExchangeBindException;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.HashMap;
import java.util.Map;


@RestController
@RequestMapping("/product")
@Validated
public class ProductController {

    private final ProductService productService;
    private static final Logger logger = LoggerFactory.getLogger(ProductController.class);

    public ProductController(ProductService productService) {
        this.productService = productService;
    }

    @PostMapping("/saveProduct")
    public Mono<ResponseEntity<Product>> addProduct(@Valid @RequestBody Mono<Product> productInfo) {
        logger.info("Received request to save product");
        return productService.saveProduct(productInfo)
                .doOnNext(product -> logger.info("Processing product: {}", product))
                .doOnSuccess(savedProduct -> logger.info("Product saved successfully: {}", savedProduct))
                .doOnError(error -> logger.error("ERROR: Failed to save product - {}", error.getMessage()))// Pass Mono<Product> directly
                .map(savedProduct -> ResponseEntity.status(HttpStatus.CREATED).body(savedProduct))
                .defaultIfEmpty(ResponseEntity.badRequest().build());
    }



    @GetMapping("/getAllProduct")
    public ResponseEntity<Flux<Product>> getAllProducts() {
        logger.info("Fetching all products");
        return ResponseEntity.ok(productService.getAllProducts());
    }
}
