package com.nisum.products_producer.controller;

import com.nisum.products_producer.dto.ResponseDto;
import com.nisum.products_producer.model.Product;
import com.nisum.products_producer.service.KafkaProducerService;
import jakarta.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import reactor.core.publisher.Mono;

import java.time.LocalDateTime;

@RestController
@RequestMapping("/api/product")
@Slf4j
public class ProductKafkaController {


    private final KafkaProducerService producerService;

    public ProductKafkaController(KafkaProducerService producerService) {
        this.producerService = producerService;
    }

//    @PostMapping("/publish")
//    public Mono<ResponseEntity<String>> publishProduct(@RequestBody @Valid Product productRequest) {
//        return producerService.sendProductToKafka(productRequest)
//                .thenReturn(ResponseEntity.ok("Product sent successfully"))
//                .onErrorResume(e -> Mono.just(ResponseEntity.badRequest().body("Failed to publish: " + e.getMessage())));
//    }
    @PostMapping("/publish")
    public Mono<ResponseEntity<ResponseDto>> publishProduct(@RequestBody @Valid Product productRequest) {
        return producerService.sendProductToKafka(productRequest)
                .thenReturn(ResponseEntity.ok(
                        new ResponseDto(
                                true,
                                "Product sent successfully",
                                productRequest,
                                LocalDateTime.now().toString(),
                                HttpStatus.OK.value()
                        )))
                .onErrorResume(e ->
                        Mono.just(ResponseEntity.badRequest().body(
                                new ResponseDto(
                                        false,
                                        "Failed to publish: " + e.getMessage(),
                                        null,
                                        LocalDateTime.now().toString(),
                                        HttpStatus.BAD_REQUEST.value()
                                ))));
    }



}
