    package com.nisum.products_producer.model;

    import jakarta.validation.constraints.DecimalMin;
    import jakarta.validation.constraints.NotEmpty;
    import jakarta.validation.constraints.NotNull;
    import jakarta.validation.constraints.Pattern;
    import lombok.AllArgsConstructor;
    import lombok.Builder;
    import lombok.Data;
    import lombok.NoArgsConstructor;
    import org.springframework.data.annotation.Id;
    import org.springframework.data.mongodb.core.index.CompoundIndex;
    import org.springframework.data.mongodb.core.index.Indexed;
    import org.springframework.data.mongodb.core.mapping.Document;

    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @Builder
    @Document(collection = "products")
    @CompoundIndex(def = "{'name': 1, 'price': 1}", unique = true)
    public class Product {

        @Id
        private String id;

        @Indexed(unique = true) // Ensure product name is unique if needed
        @NotNull
        @NotEmpty
        @Pattern(regexp = "^(?!\\d+$)[a-zA-Z0-9 ]+$", message = "Product name must contain at least one letter and can include numbers, but cannot be only numbers.")
        private String name;

        @NotNull(message = "Price cannot be null.")
        @DecimalMin(value = "0.01", inclusive = true, message = "Price must be positive.")
        private Double price;



        @Override
        public String toString() {
            return "Product{" +
                    "id='" + id + '\'' +
                    ", name='" + name + '\'' +
                    ", price=" + price +
                    '}';
        }
    }
