package com.nisum.products_producer.controller;

import com.nisum.products_producer.model.Product;
import com.nisum.products_producer.service.KafkaProducerService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.WebFluxTest;
import org.springframework.http.MediaType;
import org.springframework.test.context.bean.override.mockito.MockitoBean;
import org.springframework.test.web.reactive.server.WebTestClient;
import reactor.core.publisher.Mono;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@WebFluxTest
@ExtendWith(MockitoExtension.class)
public class ProductKafkaControllerTest {

    //@MockitoBean
    @Mock
    private KafkaProducerService producerService;

    @InjectMocks
    private ProductKafkaController productKafkaController;

    @Mock
    private WebTestClient webTestClient;

    @BeforeEach
    void setUp() {
        webTestClient = WebTestClient.bindToController(productKafkaController).build();
    }

    @Test
    void publishProduct_Success() {

        Product product = Product.builder().id("101").name("Laptop").price(1200.0).build();
        when(producerService.sendProductToKafka(any(Product.class))).thenReturn(Mono.empty());

        webTestClient.post().uri("/api/product/publish")
                .contentType(MediaType.APPLICATION_JSON)
                .bodyValue(product)
                .exchange()
                .expectStatus().isOk()
                .expectBody(String.class)
                .isEqualTo("Product sent successfully");
    }

    @Test
    void publishProduct_Failure() {

        Product product = Product.builder().id("101").name("Laptop").price(1200.0).build();
        when(producerService.sendProductToKafka(any(Product.class)))
                .thenReturn(Mono.error(new RuntimeException("Kafka error")));


        webTestClient.post().uri("/api/product/publish")
                .contentType(MediaType.APPLICATION_JSON)
                .bodyValue(product)
                .exchange()
                .expectStatus().isBadRequest()
                .expectBody(String.class)
                .isEqualTo("Failed to publish: Kafka error");
    }
}
